// (C) Copyright 2002-2007 by Autodesk, Inc. 
//
// Permission to use, copy, modify, and distribute this software in
// object code form for any purpose and without fee is hereby granted, 
// provided that the above copyright notice appears in all copies and 
// that both that copyright notice and the limited warranty and
// restricted rights notice below appear in all supporting 
// documentation.
//
// AUTODESK PROVIDES THIS PROGRAM "AS IS" AND WITH ALL FAULTS. 
// AUTODESK SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF
// MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE.  AUTODESK, INC. 
// DOES NOT WARRANT THAT THE OPERATION OF THE PROGRAM WILL BE
// UNINTERRUPTED OR ERROR FREE.
//
// Use, duplication, or disclosure by the U.S. Government is subject to 
// restrictions set forth in FAR 52.227-19 (Commercial Computer
// Software - Restricted Rights) and DFAR 252.227-7013(c)(1)(ii)
// (Rights in Technical Data and Computer Software), as applicable.
//

//-----------------------------------------------------------------------------
//----- acrxEntryPoint.cpp
//-----------------------------------------------------------------------------
#include "StdAfx.h"
#include "resource.h"
#include "SampleUtils.h"

//-----------------------------------------------------------------------------
#define szRDS _RXST("ADS")

//-----------------------------------------------------------------------------
//----- ObjectARX EntryPoint
class CConstraintModelingApp : public AcRxArxApp {

public:
	CConstraintModelingApp () : AcRxArxApp () {}

	virtual AcRx::AppRetCode On_kInitAppMsg (void *pkt) {
		// TODO: Load dependencies here

		// You *must* call On_kInitAppMsg here
		AcRx::AppRetCode retCode =AcRxArxApp::On_kInitAppMsg (pkt) ;
		
		// TODO: Add your initialization code here

		return (retCode) ;
	}

	virtual AcRx::AppRetCode On_kUnloadAppMsg (void *pkt) {
		// TODO: Add your code here

		// You *must* call On_kUnloadAppMsg here
		AcRx::AppRetCode retCode =AcRxArxApp::On_kUnloadAppMsg (pkt) ;

		// TODO: Unload dependencies here

		return (retCode) ;
	}

	virtual void RegisterServerComponents () {
		//----- Self-register COM server upon loading.
		if ( FAILED(::DllRegisterServer ()) )
			acutPrintf (_RXST("Failed to register COM server.\n")) ;
	}


	// - ADSConstraintModeling._LineParConst command (do not rename)
	static void ADSConstraintModeling_LineParConst(void)
	{
		// Command to add a parallel constraint between two lines (AcDbLine) selected
		ads_name ent1, ent2;
		ads_point pt1,pt2;
		AcDbObjectId obj1,obj2;

		// Select the AcDbLine entities to add constraints to
		acedEntSel(L"\nSelect the first Line:",ent1,pt1);
		acedEntSel(L"\nSelect the second Line:",ent2,pt2);

		acdbGetObjectId(obj1,ent1);
		acdbGetObjectId(obj2,ent2);

		AcDbBlockTable *pBlockTable;
		AcDbBlockTableRecord *pSpaceRecord;
		AcDbObjectId spaceId;

		// Get the ObjectID of the current space block table record
		spaceId = acdbHostApplicationServices()->workingDatabase()->currentSpaceId();

		Acad::ErrorStatus err;

		// Initialize the associative framework, very important
		err = AcDbAssocManager::initialize();
		ASSERT(err == Acad::eOk);

		AcDbSmartObjectPointer <AcDbEntity> pEntity1(obj1, AcDb::kForRead);
		AcDbSmartObjectPointer <AcDbEntity> pEntity2(obj2, AcDb::kForRead);

		if (pEntity1.openStatus() != Acad::eOk || pEntity2.openStatus() != Acad::eOk)
		{
			acutPrintf(_T("Error , opening object"));
			return;
		}

		AcArray<AcDbSubentId> subentIds;
		AcDbFullSubentPath subentPath1;
		AcDbFullSubentPath subentPath2;

		// Obtain the edge subentities for the first line
		err = SampleUtils::getSubentIDs(pEntity1, kEdgeSubentType, subentIds);
		ASSERT(err == Acad::eOk);
		// Choose the first edge subentity to constrain
		subentPath1 = AcDbFullSubentPath(obj1, subentIds[0]);

		subentIds.setLogicalLength(0);

		// Obtain the edge subentities for the second line
		err = SampleUtils::getSubentIDs(pEntity2, kEdgeSubentType, subentIds);
		ASSERT(err == Acad::eOk);
		// Choose the first edge subentity to constrain
		subentPath2 = AcDbFullSubentPath(obj2, subentIds[0]);

		AcDbObjectId consGrpId = SampleUtils::getConstraintGroup(spaceId,
																   true,
																   DICTIONARYKEY);

		ASSERT(!consGrpId.isNull());

		AcDbObjectPointer<AcDbAssoc2dConstraintGroup> p2dConstrGrp(consGrpId, kForWrite);
		 // Could not open constraint group for updates.
		ASSERT(p2dConstrGrp.openStatus()  == Acad::eOk );

		AcDbFullSubentPathArray aPaths;
		AcGeomConstraint *pNewConstraint = NULL;
		
		aPaths.append(subentPath1);
		aPaths.append(subentPath2);
		
		// Add the edge subentities as the constrained geometry
		err = SampleUtils::addConstrainedGeometry(p2dConstrGrp, aPaths);
		ASSERT(err == Acad::eOk);

		// Add the geometric constraint
		err = p2dConstrGrp->addGeometricalConstraint(AcGeomConstraint::kParallel, aPaths, &pNewConstraint);
		ASSERT(err == Acad::eOk);

		//Evaluate the network
		SampleUtils::evaluateNetwork();

		bool showConstBar = acedShowConstraintBar(aPaths);
	}

	// - ADSConstraintModeling._AlnDimConst command (do not rename)
	static void ADSConstraintModeling_AlnDimConst(void)
	{
		// Command to add a dimensional constraint between a line (AcDbLine) and an aligned dimension (AcDbAlignedDimension) 
		// that is associated with the line
		ads_name ent1, ent2;
		ads_point pt1,pt2;
		AcDbObjectId obj1,obj2;

		// Select the line and the dimension entity to add the constraint to
		acedEntSel(L"\nSelect the Line on which the aligned dimension exists:",ent1,pt1);
		acedEntSel(L"\nSelect the aligned dimension:",ent2,pt2);

		acdbGetObjectId(obj1,ent1);
		acdbGetObjectId(obj2,ent2);

		AcDbBlockTable *pBlockTable;
		AcDbBlockTableRecord *pSpaceRecord;
		AcDbObjectId spaceId;

		//// Get the ObjectID of the current space block table record
		//spaceId = acdbHostApplicationServices()->workingDatabase()->currentSpaceId();

		Acad::ErrorStatus err;

		// Initialize the associative framework
		err = AcDbAssocManager::initialize();
		ASSERT(err == Acad::eOk);

		AcDbSmartObjectPointer <AcDbLine> pLin(obj1, AcDb::kForRead);
		AcDbSmartObjectPointer <AcDbAlignedDimension> pAlgnDim(obj2, AcDb::kForRead);

		if (pLin.openStatus() != Acad::eOk || pAlgnDim.openStatus() != Acad::eOk)
		{
			acutPrintf(_T("Error , opening object"));
			return;
		}

		AcDbAssocConstraintType constraintType = kNoneAssocConstraintType; 
		AcDbObjectId			dimDepBodyId, dimDepId;
		AcGeMatrix3d			ucsMatrix;
		
		AcGePoint3d origin(0,0,0);
		AcGeVector3d xAxis(1,0,0), yAxis(0,1,0), zAxis(0,0,1);
		ucsMatrix.getCoordSystem(origin, xAxis, yAxis, zAxis);

		const AcDbObjectId networkId = SampleUtils::getAssocNetwork(true, NULL);
		if (networkId.isNull())
			return;

		// Add the line edge subentity as constrained geometry
		AcDbAssocPersSubentIdPE* const pAssocPersSubentIdPE
		= AcDbAssocPersSubentIdPE::cast(pLin->queryX(AcDbAssocPersSubentIdPE::desc()));

		if(NULL == pAssocPersSubentIdPE)
			return;

		AcArray<AcDbSubentId> edgeSubentIds, vertexSubentIds;

		pAssocPersSubentIdPE->getAllSubentities(pLin,
												kEdgeSubentType, 
												edgeSubentIds);

		AcDbFullSubentPath subentPath;
		subentPath = AcDbFullSubentPath(pLin->objectId(), edgeSubentIds[0]);

		AcDbObjectId consGrpId = SampleUtils::getConstraintGroup(NULL, true);

		ASSERT(!consGrpId.isNull());

		AcConstrainedGeometry* pConsGeom=NULL;

		// It is not a good idea to to open 2dConstraintGroup using a smart pointer because
		// we want to control when it closes. Its always a good idea to close it immediately after
		// usage as the assoc framework internally opens it a number of times for its own purposes
		//AcDbObjectPointer<AcDbAssoc2dConstraintGroup> p2dConstrGrp(consGrpId, kForWrite);
		AcDbAssoc2dConstraintGroup* p2dConstrGrp = NULL;
		err = acdbOpenAcDbObject((AcDbObject* &)p2dConstrGrp, consGrpId, kForWrite);
		err = p2dConstrGrp->addConstrainedGeometry(subentPath, pConsGeom);
		p2dConstrGrp->close();

		//// Work on the dimension to convert it to dimensional constraint
		//if (obj2.objectClass() != NULL && obj2.objectClass()->isDerivedFrom(AcDbDimension::desc()))
		//err = AcDbAssocDimDependencyBodyBase::getFromEntity(obj2, dimDepBodyId);

		constraintType = kDistanceAssocConstraintType; // This is the constraint type for aligned dimensions

		AcString name, expression;
		err = SampleUtils::getNameAndExpressionFromDimension(obj2, constraintType, name, expression);

		//create value dependency
		AcDbObjectId varId, varDepId;
		err = SampleUtils::addNewVariableToAssocNetwork(name, expression, networkId, varId, varDepId);

		//create dim dependency
		err = AcDbAssocDimDependencyBody::createAndPostToDatabase(obj2, dimDepId, dimDepBodyId);

		const bool bPreviousValue = AcDbAssocDimDependencyBodyBase::setEraseDimensionIfDependencyIsErased(false);

		err = SampleUtils::createDistanceConstraint(AcDistanceConstraint::kNotDirected,varDepId,dimDepId,pConsGeom,subentPath);

		AcDbAssocDimDependencyBodyBase::setEraseDimensionIfDependencyIsErased(bPreviousValue);

		//Evaluate the network
		SampleUtils::evaluateNetwork();

	}
} ;

//-----------------------------------------------------------------------------
IMPLEMENT_ARX_ENTRYPOINT(CConstraintModelingApp)

ACED_ARXCOMMAND_ENTRY_AUTO(CConstraintModelingApp, ADSConstraintModeling, _LineParConst, LineParConst, ACRX_CMD_TRANSPARENT, NULL)
ACED_ARXCOMMAND_ENTRY_AUTO(CConstraintModelingApp, ADSConstraintModeling, _AlnDimConst, AlnDimConst, ACRX_CMD_TRANSPARENT, NULL)
