// $Header: //depot/airmax/airlook/releases/2.1/source/apps/Janus/JanusRibbon/GlobalCommandHandler.cs#2 $ 
// $Change: 16969 $ $DateTime: 2008/12/08 23:05:24 $ $Author: ganesae $
// $NoKeywords: $
//////////////////////////////////////////////////////////////////////////////
//
// (C) Copyright 2008 by Autodesk, Inc.
//
// Permission to use, copy, modify, and distribute this software in
// object code form for any purpose and without fee is hereby granted, 
// provided that the above copyright notice appears in all copies and 
// that both that copyright notice and the limited warranty and
// restricted rights notice below appear in all supporting 
// documentation.
//
// AUTODESK PROVIDES THIS PROGRAM "AS IS" AND WITH ALL FAULTS. 
// AUTODESK SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF
// MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE. AUTODESK, INC. 
// DOES NOT WARRANT THAT THE OPERATION OF THE PROGRAM WILL BE
// UNINTERRUPTED OR ERROR FREE.
//
// Use, duplication, or disclosure by the U.S. Government is subject to 
// restrictions set forth in FAR 52.227-19 (Commercial Computer
// Software - Restricted Rights) and DFAR 252.227-7013(c)(1)(ii)
// (Rights in Technical Data and Computer Software), as applicable.
// 
//////////////////////////////////////////////////////////////////////////////
// Name:            GlobalCommandHandler.cs
// Description:     
//////////////////////////////////////////////////////////////////////////////

using System;
using System.Collections.Generic;
using System.Text;
using System.Xml;
using System.IO;
using System.Windows;
using System.Windows.Interop;
using System.Windows.Input;
using System.Windows.Controls;
using System.Windows.Markup;
using System.Diagnostics;
using System.Reflection;
using System.Windows.Threading;
using System.Globalization;
using System.Runtime.InteropServices;
using System.Collections.ObjectModel;
using Autodesk.Windows;
using Autodesk.Internal.Windows;
using Autodesk.Windows.ToolBars;

namespace CuiCustomControls
{
  // This is a global command handler for all the commands raised by 
  // command items in ribbon, and menu.
  internal class GlobalCommandHandler : ICommand
  {
    [DllImport("user32.dll", CharSet = CharSet.Auto, ExactSpelling = true)]
    public static extern IntPtr SetFocus(HandleRef hWnd);

    public event EventHandler CanExecuteChanged;

    public GlobalCommandHandler()
    {
      mTimer = new DispatcherTimer();
      mTimer.Tick += new EventHandler(Timer_Tick);
      mTimer.Interval = new TimeSpan(0, 0, 2);

      // Uncomment this to raise CanExecuteChanged event repeatedly 
      // at an interval.
      // mTimer.Start();
      ComponentManager.CommandHandler = this;
    }

    public bool CanExecute(object parameter)
    {
      return true;
    }

    public void Execute(object parameter)
    {
      Collection<string> propValues = new Collection<string>();

      RibbonItem item = parameter as RibbonItem;
      if (item != null)
      {
        propValues.Add(FormatPropertyValue("Type\t", parameter.GetType().FullName));
        propValues.Add(FormatPropertyValue("Id\t", item.Id));
        propValues.Add(FormatPropertyValue("Text\t", item.Text));
        propValues.Add(FormatPropertyValue("Name\t", item.Name));
        propValues.Add(FormatPropertyValue("Tag\t", (item.Tag != null) ? item.Tag.ToString() : null));
      }

      RibbonCommandItem btn = parameter as RibbonCommandItem;
      if (btn != null && !btn.IsCheckable)
        btn.IsActive = true;

      string msg = "";
      foreach (string propValue in propValues)
      {
        if (msg != "")
          msg += "\n";
        msg += propValue;
      }

      MessageBox.Show(string.Format("Global command handler invoked for the following item:\n\n{0}", msg), "Command Handler");

      if (btn != null)
        btn.IsActive = false;

      // set the focus to AutoCAD
      SetFocus(new HandleRef(null, Autodesk.AutoCAD.ApplicationServices.Application.MainWindow.Handle));
      // now pump the command in
      Autodesk.AutoCAD.ApplicationServices.Application.DocumentManager.MdiActiveDocument.SendStringToExecute(item.Tag.ToString(), true, false, true);
    }

    string FormatPropertyValue(string name, string value)
    {
      string val = null;
      if (value != null)
        val = value.Replace("\r", "\\n");
      return string.Format("{0}: {1}", name, val);
    }

    private void Timer_Tick(object sender, EventArgs e)
    {
      // Raise this event whenever something changes which affects 
      // CanExecute. Doing this will cause CanExecute() to be called 
      // for all buttons whose commands are handled by this handler. 
      if (CanExecuteChanged != null)
        CanExecuteChanged(this, new EventArgs());
    }

    DispatcherTimer mTimer;
  }
}
