﻿using System;
using System.Collections.Generic;
using System.Windows;
using Autodesk.Windows;
using System.Windows.Media.Imaging;
using Autodesk.AutoCAD.ApplicationServices;
using Autodesk.AutoCAD.DatabaseServices;
using MyUtilityFunctions;
using System.Windows.Media;
using System.Reflection;
using System.Windows.Input;

namespace CuiCustomControls
{
  #region MySplitButton
  // define my own ribbon split button
  public class MySplitButton : RibbonSplitButton, System.Windows.Input.ICommand
  {
    #region MySplitButtonImplementation
    public MySplitButton()
    {
      // initialize this event if you want to override the CUI properties directly
      this.Initialized += new EventHandler(Initialized_Event);

      // or have the cui populate the properties after you populate yours...
      /*// initialize this control
      InitializeMySplitButton();

      // now create the Ribbon buttons inside this control
      InitializeMySplitButtons();
      this.CommandHandler = this;*/
    }

    void Initialized_Event(object sender, EventArgs e)
    {
      // initialize this control
      InitializeMySplitButton();

      // now create the Ribbon buttons inside this control
      InitializeMySplitButtons();
      this.CommandHandler = this;
    }

    private void InitializeMySplitButton()
    {
      this.ListStyle = RibbonSplitButtonListStyle.Descriptive;
      this.Size = RibbonItemSize.Large;
      this.Text = (string)Module.Resources[/*MSG0*/"MySplitButtonText"];
      this.ShowText = true;
      this.Orientation = System.Windows.Controls.Orientation.Vertical;
      this.ToolTip = GenerateToolTip(/*MSG0*/"MyApplicationHelpTopic");
      this.ShowImage = true;
      // makes the default image the same as the selected item
      this.IsSynchronizedWithCurrentItem = false;
      // set the selected (smaller image)
      this.LargeImage = (BitmapImage)Module.Resources[/*MSG0*/"Image1Selected"];
    }

    internal void InitializeMySplitButtons()
    {
      // Ribbon transparency color 236, 233, 216 
      Items.Clear();
      RibbonButton item = new RibbonButton();
      item.Text = (string)Module.Resources[/*MSG0*/"MyOption1"];
      item.Image = (BitmapImage)Module.Resources[/*MSG0*/"Image1"];
      item.LargeImage = (BitmapImage)Module.Resources[/*MSG0*/"Image1"];
      Items.Add(item);
      this.Current = item;

      item = new RibbonButton();
      item.Text = (string)Module.Resources[/*MSG0*/"MyOption2"];
      item.Image = (BitmapImage)Module.Resources[/*MSG0*/"Image2"];
      item.LargeImage = (BitmapImage)Module.Resources[/*MSG0*/"Image2"];
      Items.Add(item);

      item = new RibbonButton();
      item.Text = (string)Module.Resources[/*MSG0*/"MyOption3"];
      item.Image = (BitmapImage)Module.Resources[/*MSG0*/"Image3"];
      item.LargeImage = (BitmapImage)Module.Resources[/*MSG0*/"Image3"];
      Items.Add(item);

      item = new RibbonButton();
      item.Text = (string)Module.Resources[/*MSG0*/"MyOption4"];
      item.Image = (BitmapImage)Module.Resources[/*MSG0*/"Image4"];
      item.LargeImage = (BitmapImage)Module.Resources[/*MSG0*/"Image4"];
      Items.Add(item);

      item = new RibbonButton();
      item.Text = (string)Module.Resources[/*MSG0*/"MyOption5"];
      item.Image = (BitmapImage)Module.Resources[/*MSG0*/"Image5"];
      item.LargeImage = (BitmapImage)Module.Resources[/*MSG0*/"Image5"];
      Items.Add(item);
    }

    internal static RibbonToolTip GenerateToolTip(string helpTopic)
    {
      string text = (string)Module.Resources[/*MSG0*/"MySplitButtonTooltipText"];
      string commandName = (string)Module.Resources[/*MSG0*/"MySplitButtonTooltipTextTitle"];
      RibbonToolTip tooltip = MyUtilityFunctions.Utils.GenerateRibbonToolTip(text, commandName, helpTopic);
      return tooltip;
    }
    #endregion

    #region My Ribbon Item Events
    public event EventHandler CanExecuteChanged;
    // this is called for each ribbon item, we can set whether it is allowed to call execute or not
    public bool CanExecute(object parameter)
    {
      return true;
    }
    // this command is executed when the ribbon item is selected
    public void Execute(object parameter)
    {
      RibbonButton ribbonButton = parameter as RibbonButton;
      System.Windows.MessageBox.Show(ribbonButton.Text + " was selected");

      // now loop and find which item was selected
      for (int i = 1; i <= 5; ++i)
      {
        // create the resource key for the button text
        string key = "MyOption" + i.ToString();
        // extract the button text from the resources
        string keyText = (string)Module.Resources[key];
        // now is it the one we want..?
        if (keyText == ribbonButton.Text)
        {
          // extract the selected text from the resources.
          string imageTextResource = "Image" + i.ToString() + "Selected";
          // now set the image
          this.LargeImage = (BitmapImage)Module.Resources[imageTextResource];
          break;
        }
      }
    }
    // To suppress build warning
    void RaiseCanExecuteChanged()
    {
      if (CanExecuteChanged != null)
        CanExecuteChanged(this, new EventArgs());
    }
    #endregion
  }
  #endregion
}
