// (C) Copyright 2002-2007 by Autodesk, Inc. 
//
// Permission to use, copy, modify, and distribute this software in
// object code form for any purpose and without fee is hereby granted, 
// provided that the above copyright notice appears in all copies and 
// that both that copyright notice and the limited warranty and
// restricted rights notice below appear in all supporting 
// documentation.
//
// AUTODESK PROVIDES THIS PROGRAM "AS IS" AND WITH ALL FAULTS. 
// AUTODESK SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF
// MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE.  AUTODESK, INC. 
// DOES NOT WARRANT THAT THE OPERATION OF THE PROGRAM WILL BE
// UNINTERRUPTED OR ERROR FREE.
//
// Use, duplication, or disclosure by the U.S. Government is subject to 
// restrictions set forth in FAR 52.227-19 (Commercial Computer
// Software - Restricted Rights) and DFAR 252.227-7013(c)(1)(ii)
// (Rights in Technical Data and Computer Software), as applicable.
//

//-----------------------------------------------------------------------------
//----- ADSAssocCustEnt.cpp : Implementation of ADSAssocCustEnt
//-----------------------------------------------------------------------------
#include "StdAfx.h"
#include "ADSAssocCustEnt.h"
#define _USE_MATH_DEFINES
#include "math.h"

//-----------------------------------------------------------------------------
Adesk::UInt32 ADSAssocCustEnt::kCurrentVersionNumber =1 ;

//-----------------------------------------------------------------------------
ACRX_DXF_DEFINE_MEMBERS (
	ADSAssocCustEnt, AcDbEntity,
	AcDb::kDHL_CURRENT, AcDb::kMReleaseCurrent, 
	AcDbProxyEntity::kNoOperation, ADSASSOCCUSTENT,
	"ADSASSOCCUSTOMENTITYAPP"
	"|Product Desc:     A description for your object"
	"|Company:          Your company name"
	"|WEB Address:      Your company WEB site address"
)

//-----------------------------------------------------------------------------
ADSAssocCustEnt::ADSAssocCustEnt () : AcDbEntity () , m_location(0,0,0)
{
}

ADSAssocCustEnt::~ADSAssocCustEnt () {
}

//-----------------------------------------------------------------------------
//----- AcDbObject protocols
//- Dwg Filing protocol
Acad::ErrorStatus ADSAssocCustEnt::dwgOutFields (AcDbDwgFiler *pFiler) const {
	assertReadEnabled () ;
	//----- Save parent class information first.
	Acad::ErrorStatus es =AcDbEntity::dwgOutFields (pFiler) ;
	if ( es != Acad::eOk )
		return (es) ;
	//----- Object version number needs to be saved first
	if ( (es =pFiler->writeUInt32 (ADSAssocCustEnt::kCurrentVersionNumber)) != Acad::eOk )
		return (es) ;
	//----- Output params
	pFiler->writePoint3d(m_location);
	//.....

	return (pFiler->filerStatus ()) ;
}

Acad::ErrorStatus ADSAssocCustEnt::dwgInFields (AcDbDwgFiler *pFiler) {
	assertWriteEnabled () ;
	//----- Read parent class information first.
	Acad::ErrorStatus es =AcDbEntity::dwgInFields (pFiler) ;
	if ( es != Acad::eOk )
		return (es) ;
	//----- Object version number needs to be read first
	Adesk::UInt32 version =0 ;
	if ( (es =pFiler->readUInt32 (&version)) != Acad::eOk )
		return (es) ;
	if ( version > ADSAssocCustEnt::kCurrentVersionNumber )
		return (Acad::eMakeMeProxy) ;
	//- Uncomment the 2 following lines if your current object implementation cannot
	//- support previous version of that object.
	//if ( version < ADSAssocCustEnt::kCurrentVersionNumber )
	//	return (Acad::eMakeMeProxy) ;
	//----- Read params
	pFiler->readPoint3d(&m_location);
	//.....

	return (pFiler->filerStatus ()) ;
}

//-----------------------------------------------------------------------------
//----- AcDbEntity protocols
Adesk::Boolean ADSAssocCustEnt::subWorldDraw (AcGiWorldDraw *mode) {
	assertReadEnabled () ;

	mode->subEntityTraits().setSelectionMarker(1);
	mode->geometry().circle(m_location,5,AcGeVector3d::kZAxis);

	AcGeVector3d vec = AcGeVector3d::kXAxis;

	vec.rotateBy(M_PI_4,AcGeVector3d::kZAxis);
	mode->subEntityTraits().setSelectionMarker(2);
	mode->geometry().circle(m_location+vec*2.5,1,AcGeVector3d::kZAxis);

	vec.rotateBy(M_PI_2,AcGeVector3d::kZAxis);
	mode->subEntityTraits().setSelectionMarker(3);
	mode->geometry().circle(m_location+vec*2.5,1,AcGeVector3d::kZAxis);

	vec.rotateBy(M_PI_2,AcGeVector3d::kZAxis);
	mode->subEntityTraits().setSelectionMarker(4);
	mode->geometry().circularArc(m_location,3,AcGeVector3d::kZAxis,vec,M_PI_2);

	return (AcDbEntity::subWorldDraw (mode)) ;
}


Adesk::UInt32 ADSAssocCustEnt::subSetAttributes (AcGiDrawableTraits *traits) {
	assertReadEnabled () ;
	return (AcDbEntity::subSetAttributes (traits)) ;
}



ErrorStatus ADSAssocCustEnt::SetPosition(const AcGePoint3d pos)
{
	assertWriteEnabled();
	ErrorStatus es=Acad::eOk;

	m_location=pos;

	return es;
}

AcGePoint3d ADSAssocCustEnt::GetPosition(void)
{
	assertReadEnabled();
	return m_location;
}

// -----------------------------------------------------------------------------
Acad::ErrorStatus ADSAssocCustEnt::subGetSubentPathsAtGsMarker(AcDb::SubentType type, Adesk::GsMarker gsMark, const AcGePoint3d & pickPoint, const AcGeMatrix3d & viewXform, int & numPaths, AcDbFullSubentPath *& subentPaths, int numInserts, AcDbObjectId * entAndInsertStack) const
{
	//Acad::ErrorStatus retCode =AcDbEntity::getSubentPathsAtGsMarker (type, gsMark, pickPoint, viewXform, numPaths, subentPaths, numInserts, entAndInsertStack) ;
	assertReadEnabled();

    AcDbFullSubentPath *pPaths = new AcDbFullSubentPath[1];	
    pPaths->subentId().setIndex(gsMark);
    pPaths->subentId().setType(AcDb::kClassSubentType);
    pPaths->subentId().setTypeClass(ADSAssocCustEnt::desc());

    pPaths->objectIds().append(this->objectId());

    subentPaths = pPaths;
    numPaths = 1;

    return (Acad::eOk);
	//return (retCode) ;
}

// -----------------------------------------------------------------------------
Acad::ErrorStatus ADSAssocCustEnt::subGetGsMarkersAtSubentPath(const AcDbFullSubentPath & subPath, AcArray<Adesk::GsMarker> & gsMarkers) const
{
	/*Acad::ErrorStatus retCode =AcDbEntity::getGsMarkersAtSubentPath (subPath, gsMarkers) ;
	return (retCode) ;*/
	assertReadEnabled () ;

	AcDbSubentId subEntId = subPath.subentId();
	Adesk::GsMarker gsMarker( subEntId.index() );
    gsMarkers.append(gsMarker);

    return (Acad::eOk);	
}

// -----------------------------------------------------------------------------
AcDbEntity * ADSAssocCustEnt::subSubentPtr(const AcDbFullSubentPath & id) const
{
	/*AcDbEntity * retCode =AcDbEntity::subentPtr (id) ;
	return (retCode) ;*/
	assertReadEnabled () ;
		
	if(1 == id.subentId().index())
    {
        return new AcDbCircle(m_location,AcGeVector3d::kZAxis, 5);			
    }
    else if(2 == id.subentId().index())
    {
		AcGeVector3d vec = AcGeVector3d::kXAxis;
		vec.rotateBy(M_PI_4,AcGeVector3d::kZAxis);
        return new AcDbCircle(m_location+vec*3,AcGeVector3d::kZAxis, 1);

    }else if(3 == id.subentId().index())
    {
        AcGeVector3d vec = AcGeVector3d::kXAxis;
		vec.rotateBy(M_PI_4+M_PI_2,AcGeVector3d::kZAxis);
        return new AcDbCircle(m_location+vec*3,AcGeVector3d::kZAxis, 1);
    }
	else if(4 == id.subentId().index())
    {
        AcGeVector3d vec = AcGeVector3d::kXAxis;
		vec.rotateBy(M_PI_4+M_PI_2,AcGeVector3d::kZAxis);
        return new AcDbArc(m_location,3,M_PI+M_PI_4,2*M_PI-M_PI_4);
    }

    return (NULL) ;
}

// -----------------------------------------------------------------------------
Acad::ErrorStatus ADSAssocCustEnt::getPlane(AcGePlane & param2, AcDb::Planarity & param3) const
{
	assertReadEnabled();

    param3 = kPlanar;
	param2.set(m_location,AcGeVector3d::kZAxis);

    return eOk;
}
