#include "StdAfx.h"
#include "ADSAssocCustEntPE.h"
#include "ADSAssocCustEnt.h"
#define _USE_MATH_DEFINES
#include "math.h"

ACRX_CONS_DEFINE_MEMBERS   (ADSAssocCustEntPE,           AcDbAssocPersSubentIdPE, 0)

//ADSAssocCustEntPE::ADSAssocCustEntPE(void)
//{
//}
//
//ADSAssocCustEntPE::~ADSAssocCustEntPE(void)
//{
//}

const Adesk::GsMarker kStartVertexSubentIndex  = -1;
const Adesk::GsMarker kStart2VertexSubentIndex = -2;
const Adesk::GsMarker kEndVertexSubentIndex    = -3;
const Adesk::GsMarker kEnd2VertexSubentIndex   = -4;
const Adesk::GsMarker kCenterVertexSubentIndex = -5;
const Adesk::GsMarker kChinVertexSubentIndex    = -6;
const Adesk::GsMarker kStartEdgeSubentIndex    = -1;
const Adesk::GsMarker kEndEdgeSubentIndex      = -2;
const Adesk::GsMarker kSingleEdgeSubentIndex   =  1; // Matches with AcDbAssocSingleEdgePersSubentId
const Adesk::GsMarker kPointVertexSubentIndex  = -1;
const Adesk::GsMarker kMaxNegativeSubentIndex  = -1000;

// The subentity IDs below exemplify how the subentities are identified though we just use the 
// kCenterVertexSubentId and kChinVertexSubentId in this sample
const AcDbSubentId ADSAssocCustEntPE::kStartVertexSubentId     (kVertexSubentType, kStartVertexSubentIndex);
const AcDbSubentId ADSAssocCustEntPE::kStart2VertexSubentId    (kVertexSubentType, kStart2VertexSubentIndex);
const AcDbSubentId ADSAssocCustEntPE::kEndVertexSubentId       (kVertexSubentType, kEndVertexSubentIndex);
const AcDbSubentId ADSAssocCustEntPE::kEnd2VertexSubentId      (kVertexSubentType, kEnd2VertexSubentIndex);
const AcDbSubentId ADSAssocCustEntPE::kCenterVertexSubentId    (kVertexSubentType, kCenterVertexSubentIndex);
const AcDbSubentId ADSAssocCustEntPE::kChinVertexSubentId       (kVertexSubentType, kChinVertexSubentIndex);
const AcDbSubentId ADSAssocCustEntPE::kStartEdgeSubentId       (kEdgeSubentType,   kStartEdgeSubentIndex);
const AcDbSubentId ADSAssocCustEntPE::kEndEdgeSubentId         (kEdgeSubentType,   kEndEdgeSubentIndex);
const AcDbSubentId ADSAssocCustEntPE::kSingleEdgeSubentId      (kEdgeSubentType,   kSingleEdgeSubentIndex);



//////////////////////////////////////////////////////////////////////////////
//                     ADSAssocCustEntPE
//////////////////////////////////////////////////////////////////////////////

// This method is one of the most commonly called methods by the associative framework
Acad::ErrorStatus 
ADSAssocCustEntPE::getAllSubentities(const AcDbEntity*    /*pEntity*/,
                                                 AcDb::SubentType       subentType,
                                                 AcArray<AcDbSubentId>& allSubentIds)
{
    allSubentIds.setLogicalLength(0);

    switch (subentType)
    {
    case kVertexSubentType:
        {
            allSubentIds.setLogicalLength(2);
            allSubentIds[0] = kCenterVertexSubentId;
			allSubentIds[1] = kChinVertexSubentId;
        }
        break;
    case kEdgeSubentType:
        {
			// This code is commented. But if one inherits the custom entity from
			// AcDbCurve based entities, specifying an edge subentity (like below)
			// helps

            //allSubentIds.setLogicalLength(1);
            //allSubentIds[0] = kSingleEdgeSubentId;
        }
        break;
    default:
        // There are no subentities of other types
        break;
    }
    return eOk;
}


// Get the vertices associated with an edge
Acad::ErrorStatus 
ADSAssocCustEntPE::getEdgeVertexSubentities(const AcDbEntity*    /*pEntity*/,
                                                        const AcDbSubentId&  /*edgeSubentId*/,
                                                        AcDbSubentId&          startVertexSubentId,
                                                        AcDbSubentId&          endVertexSubentId,
                                                        AcArray<AcDbSubentId>& otherVertexSubentIds)
{
    otherVertexSubentIds.setLogicalLength(2);
    startVertexSubentId     = kNullSubentId;
    endVertexSubentId       = kNullSubentId;
    otherVertexSubentIds[0] = kCenterVertexSubentId;
	otherVertexSubentIds[1] = kChinVertexSubentId;
	//otherVertexSubentIds[0] = kNullSubentId;
    return eOk;
}

// Get the vertex geometry associated with a vertex subentity
Acad::ErrorStatus 
ADSAssocCustEntPE::getVertexSubentityGeometry(const AcDbEntity*   pEntity,
                                                          const AcDbSubentId& vertexSubentId,
                                                          AcGePoint3d&        vertexPosition)
{
    ADSAssocCustEnt* const pCurve = ADSAssocCustEnt::cast(pEntity);

    if (pCurve == NULL)
    {
        ASSERT(!"ADSAssocCustEnt expected");
        return eNotThatKindOfClass;
    }

	if(kCenterVertexSubentId == vertexSubentId)
		vertexPosition = pCurve->GetPosition();
	else if(kChinVertexSubentId == vertexSubentId)
	{
		AcGePoint3d chin = pCurve->GetPosition();
		chin += -AcGeVector3d::kYAxis*5;
		vertexPosition = chin;
	}

    return eOk;
}

// Update the entity with the new vertex geometry information
Acad::ErrorStatus 
ADSAssocCustEntPE::setVertexSubentityGeometry(AcDbEntity*         pEntity,
                                                          const AcDbSubentId& vertexSubentId,
                                                          const AcGePoint3d&  newVertexPosition)
{
    ADSAssocCustEnt* const pCurve = ADSAssocCustEnt::cast(pEntity);

    if (pCurve == NULL)
    {
        ASSERT(!"ADSAssocCustEnt expected");
        return eNotThatKindOfClass;
    }

    //return pCurve->SetPosition(newVertexPosition);

	if(kCenterVertexSubentId == vertexSubentId)
		return pCurve->SetPosition(newVertexPosition);
	else if(kChinVertexSubentId == vertexSubentId)
	{
		AcGePoint3d chin = pCurve->GetPosition();
		chin += -AcGeVector3d::kYAxis*5;
		
		AcGeVector3d off = newVertexPosition - chin;

		return pCurve->SetPosition(pCurve->GetPosition()+off);

	}
}


// Update the entity with the new edge geometry information
Acad::ErrorStatus 
ADSAssocCustEntPE::setEdgeSubentityGeometry(AcDbEntity*         pEntity,
                                                        const AcDbSubentId& /*edgeSubentId*/,
                                                        const AcGeCurve3d*  pNewEdgeCurve)
{
    ADSAssocCustEnt* const pCurve = ADSAssocCustEnt::cast(pEntity);

    if (pCurve == NULL)
    {
        ASSERT(!"ADSAssocCustEnt expected");
        return eNotThatKindOfClass;
    }

    if (pNewEdgeCurve == NULL || !pNewEdgeCurve->isKindOf(AcGe::kCircArc3d))
    {
        ASSERT(!"AcGeCircArc3d expected");
        return eInvalidInput;
    }

    const AcGeCircArc3d* const pGeCurve = static_cast<const AcGeCircArc3d*>(pNewEdgeCurve);
    ASSERT(pGeCurve != NULL);

    pCurve->SetPosition(pGeCurve->center());
    /*pCurve->setNormal(pGeCurve->normal());
    pCurve->setRadius(pGeCurve->radius());*/
    return eOk;
}

Acad::ErrorStatus
ADSAssocCustEntPE::getSubentTransFromStretchPoints(const AcDbEntity*   pEntity,
                                                               const AcDbIntArray& indices, 
                                                               const AcGeVector3d& offset,
                                                               AcArray<AcDbAssocSubentTransInfo>& aSubentTransInfos) const
{
  	return eNotImplemented;
}

Acad::ErrorStatus 
ADSAssocCustEntPE::getSubentTransFromGripPoints(const AcDbEntity*   pEntity,
                                                            const AcDbVoidPtrArray& gripAppData, 
                                                            const AcGeVector3d& offset,
                                                            AcArray<AcDbAssocSubentTransInfo>& aSubentTransInfos) const
{
  	return eNotImplemented;
}

// Get the edge geometry as an AcGeCurve3d
Acad::ErrorStatus ADSAssocCustEntPE::getEdgeSubentityGeometry(const AcDbEntity*  pEntity,
                                                      const AcDbSubentId& /*edgeSubentId*/,
                                                      AcGeCurve3d*&       pEdgeCurve)
{
	Acad::ErrorStatus es = Acad::eOk;

	ADSAssocCustEnt* const pCustEnt = ADSAssocCustEnt::cast(pEntity);

      if (pCustEnt == NULL)
      {
        ASSERT(!/*MSG0*/"ADSAssocCustEnt expected");
        return eNotThatKindOfClass;
      }

	  pEdgeCurve = new AcGeCircArc3d(pCustEnt->GetPosition(),AcGeVector3d::kZAxis,5);
	  //pEdgeCurve = new AcGeCircArc3d(pCustEnt->GetPosition(),AcGeVector3d::kZAxis,AcGeVector3d::kXAxis,5,0,2*M_PI*359/360);
      

	return es;
}