#pragma once
#include "AcDbAssocPersSubentIdPE.h"

#ifdef ASSOCCUSTOMENTITY_MODULE
#define DLLIMPEXP __declspec(dllexport)
#else
//----- Note: we don't use __declspec(dllimport) here, because of the
//----- "local vtable" problem with msvc. If you use __declspec(dllimport),
//----- then, when a client dll does a new on the class, the object's
//----- vtable pointer points to a vtable allocated in that client
//----- dll. If the client dll then passes the object to another dll,
//----- and the client dll is then unloaded, the vtable becomes invalid
//----- and any virtual calls on the object will access invalid memory.
//-----
//----- By not using __declspec(dllimport), we guarantee that the
//----- vtable is allocated in the server dll during the ctor and the
//----- client dll does not overwrite the vtable pointer after calling
//----- the ctor. And, since we expect the server dll to remain in
//----- memory indefinitely, there is no problem with vtables unexpectedly
//----- going away.
#define DLLIMPEXP
#endif


class DLLIMPEXP ADSAssocCustEntPE :	public AcDbAssocPersSubentIdPE
{
public:
	/*ADSAssocCustEntPE(void);
	~ADSAssocCustEntPE(void);*/

	ACRX_DECLARE_MEMBERS(ADSAssocCustEntPE);

	// Common vertex and edge AcDbSubentIds for subentities that are 
    // available on most curve-based entities
    //
    static const AcDbSubentId kStartVertexSubentId;  // Start vertex of a curve
    static const AcDbSubentId kStart2VertexSubentId; // Second vertex from the start (for polylines)
    static const AcDbSubentId kEndVertexSubentId;    // End vertex of a curve
    static const AcDbSubentId kEnd2VertexSubentId;   // Second vertex from the end (for polylines)
    static const AcDbSubentId kCenterVertexSubentId; // Center vertex of an arc curve segment
    static const AcDbSubentId kChinVertexSubentId;    // midpoint vertex of an arc or line curve segment
    static const AcDbSubentId kStartEdgeSubentId;    // Start edge of a polyline
    static const AcDbSubentId kEndEdgeSubentId;      // End edge of a polyline
    static const AcDbSubentId kSingleEdgeSubentId;   // Single edge when there is just a single curve

    virtual Acad::ErrorStatus getAllSubentities(const AcDbEntity*      pEntity,
                                                AcDb::SubentType       subentType,
                                                AcArray<AcDbSubentId>& allSubentIds);

    virtual Acad::ErrorStatus getEdgeVertexSubentities(const AcDbEntity*      pEntity,
                                                       const AcDbSubentId&    edgeSubentId,
                                                       AcDbSubentId&          startVertexSubentId,
                                                       AcDbSubentId&          endVertexSubentId,
                                                       AcArray<AcDbSubentId>& otherVertexSubentIds);

    virtual Acad::ErrorStatus getVertexSubentityGeometry(const AcDbEntity*   pEntity,
                                                         const AcDbSubentId& vertexSubentId,
                                                         AcGePoint3d&        vertexPosition);

    virtual Acad::ErrorStatus setVertexSubentityGeometry(AcDbEntity*         pEntity,
                                                         const AcDbSubentId& vertexSubentId,
                                                         const AcGePoint3d&  newVertexPosition);

	virtual Acad::ErrorStatus getEdgeSubentityGeometry(const AcDbEntity*  pEntity,
                                                      const AcDbSubentId& /*edgeSubentId*/,
                                                      AcGeCurve3d*&       pEdgeCurve);

    virtual Acad::ErrorStatus setEdgeSubentityGeometry(AcDbEntity*         pEntity,
                                                       const AcDbSubentId& edgeSubentId,
                                                       const AcGeCurve3d*  pNewEdgeCurve);

    virtual Acad::ErrorStatus getSubentTransFromStretchPoints(const AcDbEntity*   pEntity,
                                                              const AcDbIntArray& indices, 
                                                              const AcGeVector3d& offset,
                                                              AcArray<AcDbAssocSubentTransInfo>& aSubentTransInfos) const;

    virtual Acad::ErrorStatus getSubentTransFromGripPoints(const AcDbEntity*   pEntity,
                                                           const AcDbVoidPtrArray& gripAppData, 
                                                           const AcGeVector3d& offset,
                                                           AcArray<AcDbAssocSubentTransInfo>& aSubentTransInfos) const;

	// virtual int getRigidSetState(const AcDbEntity* pEntity) { return kScalableRigidSet; }
};

#ifdef ASSOCCUSTOMENTITY_MODULE
ACDB_REGISTER_OBJECT_ENTRY_AUTO(ADSAssocCustEntPE)
#endif