#include "StdAfx.h"
#include "AssocSampleUtils.h"
#include "ConstraintCallBack.h"

AssocSampleUtils::AssocSampleUtils(void)
{
}

AssocSampleUtils::~AssocSampleUtils(void)
{
}


AcDbObjectId AssocSampleUtils::getConstraintGroup(AcDbDatabase  *pDatabase,
                                              bool  createIfDoesNotExist,
                                              const AcString& dictionaryKey)
{
	AcDbObjectId idConstrGroup = AcDbObjectId::kNull;
    AcDbObjectId idNetwork = 
        AcDbAssocNetwork::getInstanceFromDatabase(pDatabase,
                                                  createIfDoesNotExist,
                                                  dictionaryKey);

    if (idNetwork.isNull()) {
        ASSERT(!createIfDoesNotExist);
        return idConstrGroup; // AcDbObjectId::kNull;
    }

     // get the current UCS plane
     AcGeMatrix3d ucsMatrix;
     acedGetCurrentUCS(ucsMatrix);
     AcGePoint3d origin;
     AcGeVector3d xAxis, yAxis, zAxis;
     ucsMatrix.getCoordSystem(origin, xAxis, yAxis, zAxis);
     AcGePlane currentPlane(origin, xAxis, yAxis);

    // Try to find the constraint group in the associative network
	 if (idConstrGroup.isNull() && createIfDoesNotExist)
	 {
        AcDbObjectPointer<AcDbAssocNetwork> pNetwork(idNetwork, kForRead);
        if (eOk != pNetwork.openStatus()) {
            ASSERT(false);
            return idConstrGroup; // AcDbObjectId::kNull;
        }

		AcRxClass *pBaseClass = AcDbAssoc2dConstraintGroup::desc();
        const AcDbObjectIdArray& actionsInNetwork = pNetwork->getActions();
        for (int nCount = 0; nCount < actionsInNetwork.length(); nCount++)
        {
			const AcDbObjectId& idAction = actionsInNetwork[nCount];
            if (idAction == AcDbObjectId::kNull)
                continue;
			if (actionsInNetwork[nCount].objectClass() == NULL || 
				!actionsInNetwork[nCount].objectClass()->isDerivedFrom(pBaseClass))
				continue;
			AcDbObjectPointer<AcDbAssocAction> pAction(idAction, kForRead);
		    if (eOk != pAction.openStatus())
				continue;
			const AcDbAssoc2dConstraintGroup* const pConstGrp = 
				static_cast<AcDbAssoc2dConstraintGroup*>(pAction.object());
			ASSERT(pConstGrp);
			if (!pConstGrp)
				continue;

			AcGePlane constGrpPlane = pConstGrp->getWorkPlane();
			if (constGrpPlane.isCoplanarTo(currentPlane))
			{
                // We have an constarint group which is built on same UCS
				return idAction;
			}
		}
    }

    if (idConstrGroup.isNull() && createIfDoesNotExist)
    {
        AcDbObjectPointer<AcDbAssocNetwork> pNetwork(idNetwork, kForWrite);
        if (eOk != pNetwork.openStatus()) {
            ASSERT(false);
            return AcDbObjectId::kNull;
        }

        AcDbAssoc2dConstraintGroup* const pConstrGroup =
            new AcDbAssoc2dConstraintGroup(currentPlane);

        AcDbDatabase *pDatabase = pNetwork->database();
        if (eOk != pDatabase->addAcDbObject(idConstrGroup, pConstrGroup))
		{
            assert(false);
            delete pConstrGroup;
            return AcDbObjectId::kNull;
        }

        pConstrGroup->close();

        if (eOk != pNetwork->addAction(idConstrGroup, true)) {
            ASSERT(false);
            return AcDbObjectId::kNull;
        }
    }

    return idConstrGroup;
}


void AssocSampleUtils::GetAPathsAllNew(const AcDbEntity *pEnt, AcDb::SubentType subentType, AcDbFullSubentPathArray &aPaths,AcArray<AcConstrainedGeometry*> &pConsGeoms)
{
	AcDbFullSubentPathArray entPath;
	
	ErrorStatus err;
	AcConstrainedGeometry* pConsGeom = NULL;

	// Get the Protocol extension associated with the entity
	AcDbAssocPersSubentIdPE* const pAssocPersSubentIdPE
		= AcDbAssocPersSubentIdPE::cast(pEnt->queryX(AcDbAssocPersSubentIdPE::desc()));

	AcArray<AcDbSubentId> subentIds1, subentIds;

	// Get all the subentities associated with the subentity type such as the vertices or edges
	pAssocPersSubentIdPE->getAllSubentities(pEnt,
										   subentType,
										   subentIds1);
	
	AcDbFullSubentPath subentPath;
	int i = 0;
	//Get subent path and put them into aPaths collection
	while(i < subentIds1.length())
	{
		subentPath = AcDbFullSubentPath(pEnt->objectId(), subentIds1[i++]);
		aPaths.append(subentPath);
	}

}

void AssocSampleUtils::AddConstrainedGeometry(AcDbFullSubentPathArray &aPaths,AcArray<AcConstrainedGeometry*> &pConsGeoms)
{

	ErrorStatus err;
	
	// Get the constraint Group. This object is like the manager for all native AutoCAD constraints
	AcDbDatabase *pDatabase = acdbHostApplicationServices()->workingDatabase();
	const AcString dictionaryKey(DICTIONARYKEY);
	AcDbObjectId consGrpId = AssocSampleUtils::getConstraintGroup(pDatabase,
															   true,
															   dictionaryKey);
	if( consGrpId.isNull() )
		return;

	AcDbObjectPointer<AcDbAssoc2dConstraintGroup> p2dConstrGrp(consGrpId, kForWrite);

	// Could not open constraint group for updates.
	if (p2dConstrGrp.openStatus()  != Acad::eOk )
		return;// Acad::eInvalidInput;

	int i = 0;
	AcConstrainedGeometry* pConsGeom = NULL;
	//Add the relevent subentities as the constrained geometry
	while(i < aPaths.length())
	{
		err = p2dConstrGrp->addConstrainedGeometry(aPaths[i++], pConsGeom);
		pConsGeoms.append(pConsGeom);
	}

}

void AssocSampleUtils::AddGeomConstraint(AcGeomConstraint::GeomConstraintType typ, AcDbFullSubentPathArray &aPaths)
{
	ErrorStatus err;

	// Get the 2D constraint Group
	AcDbDatabase *pDatabase = acdbHostApplicationServices()->workingDatabase();
	const AcString dictionaryKey(DICTIONARYKEY);
	AcDbObjectId consGrpId = AssocSampleUtils::getConstraintGroup(pDatabase,
															   true,
															   dictionaryKey);
	if( consGrpId.isNull() )
		return;

	AcDbObjectPointer<AcDbAssoc2dConstraintGroup> p2dConstrGrp(consGrpId, kForWrite);

	// Add the constraint to the selected entities
	AcGeomConstraint **ppNewConstraint=NULL;
	err = p2dConstrGrp->addGeometricalConstraint(typ, aPaths, ppNewConstraint);
	
	if ( err != Acad::eOk )
		return;

	// Evaluate the constraint. This is like a refresh to make sure the constraint takes effect.
	ConstraintEvaluationCallback callback;

	AcDbObjectId idNetwork = 
	AcDbAssocNetwork::getInstanceFromDatabase(pDatabase,
											  true,
											  dictionaryKey);
	
	if (idNetwork.isNull())    
		return;// Acad::eInvalidInput; 

	AcDbObjectPointer<AcDbAssocNetwork> pNetwork(idNetwork, kForWrite);
	 if (eOk != pNetwork.openStatus())
		 return;// Acad::eInvalidInput;

	pNetwork->evaluate(&callback);
}